"""
Decorators for the authentication app.
We don't want (yet) to enforce the OIDC access token to be "fresh" for all
views, so we provide a decorator to refresh the access token only when needed.
"""

from django.utils.decorators import decorator_from_middleware

from .middleware import RefreshOIDCAccessToken

refresh_oidc_access_token = decorator_from_middleware(RefreshOIDCAccessToken)



"""Marketing backends module."""

from dataclasses import dataclass


@dataclass
class ContactData:
    """Contact data for marketing service integration."""

    email: str
    attributes: dict[str, str] | None = None
    list_ids: list[int] | None = None
    update_enabled: bool = True



# Generated by Django 5.2.3 on 2025-12-04 10:49

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('malware_detection', '0003_malwaredetection_error_code_and_more'),
    ]

    operations = [
        migrations.AddField(
            model_name='malwaredetection',
            name='file_hash',
            field=models.CharField(blank=True, help_text='hash of the file', max_length=255),
        ),
    ]



"""Marketing tasks module."""

from celery import shared_task

from lasuite.marketing import marketing
from lasuite.marketing.backends import ContactData


@shared_task
def create_or_update_contact(
    email: str,
    attributes: dict[str, str] | None = None,
    list_ids: list[int] | None = None,
    update_enabled: bool = True,
    timeout: int = None,
):
    """Create or update a contact."""
    contact_data = ContactData(email=email, attributes=attributes, list_ids=list_ids, update_enabled=update_enabled)
    return marketing.create_or_update_contact(contact_data, timeout)



"""Authentication Backends for OIDC."""

import logging
from functools import lru_cache
from json import JSONDecodeError

import jwt
import requests
from cryptography.fernet import Fernet
from django.conf import settings
from django.core.exceptions import SuspiciousOperation
from django.utils.http import parse_header_parameters
from django.utils.translation import gettext_lazy as _
from mozilla_django_oidc.auth import (
    OIDCAuthenticationBackend as MozillaOIDCAuthenticationBackend,
)
from mozilla_django_oidc.utils import import_from_settings

from lasuite.oidc_login.enums import OIDCUserEndpointFormat

logger = logging.getLogger(__name__)


@lru_cache(maxsize=1)
def get_cipher_suite():
    """Return a Fernet cipher suite."""
    key = import_from_settings("OIDC_STORE_REFRESH_TOKEN_KEY", None)
    if not key:
        raise ValueError("OIDC_STORE_REFRESH_TOKEN_KEY setting is required.")
    return Fernet(key)


def store_oidc_refresh_token(session, refresh_token):
    """Store the encrypted OIDC refresh token in the session if enabled in settings."""
    if import_from_settings("OIDC_STORE_REFRESH_TOKEN", False):
        encrypted_token = get_cipher_suite().encrypt(refresh_token.encode())
        session["oidc_refresh_token"] = encrypted_token.decode()


def get_oidc_refresh_token(session):
    """Retrieve and decrypt the OIDC refresh token from the session."""
    encrypted_token = session.get("oidc_refresh_token")
    if encrypted_token:
        return get_cipher_suite().decrypt(encrypted_token.encode()).decode()
    return None


def store_tokens(session, access_token, id_token, refresh_token):
    """Store tokens in the session if enabled in settings."""
    if import_from_settings("OIDC_STORE_ACCESS_TOKEN", False):
        session["oidc_access_token"] = access_token

    if import_from_settings("OIDC_STORE_ID_TOKEN", False):
        session["oidc_id_token"] = id_token

    store_oidc_refresh_token(session, refresh_token)


class OIDCAuthenticationBackend(MozillaOIDCAuthenticationBackend):
    """
    Custom OpenID Connect (OIDC) Authentication Backend.

    This class overrides the default OIDC Authentication Backend to accommodate differences
    in the User model, and handles signed and/or encrypted UserInfo response.
    """

    def __init__(self, *args, **kwargs):
        """
        Initialize the OIDC Authentication Backend.

        Adds an internal attribute to store the token_info dictionary.
        The purpose of `self._token_info` is to not duplicate code from
        the original `authenticate` method.
        This won't be needed after https://github.com/mozilla/mozilla-django-oidc/pull/377
        is merged.

        Sets the `OIDC_OP_USER_ENDPOINT_FORMAT` based on the settings, with fallback on "auto" mode.
        This allows to enforce specific behavior for the user endpoint format (if you want to enforce
        JWT use or JSON use).
        """
        super().__init__(*args, **kwargs)
        self._token_info = None

        self.OIDC_OP_USER_ENDPOINT_FORMAT = OIDCUserEndpointFormat[
            self.get_settings(
                "OIDC_OP_USER_ENDPOINT_FORMAT",
                OIDCUserEndpointFormat.AUTO.name,
            )
        ]
        self.OIDC_USERINFO_FULLNAME_FIELDS = self.get_settings(
            "OIDC_USERINFO_FULLNAME_FIELDS",
            [],
        )
        self.OIDC_USERINFO_ESSENTIAL_CLAIMS = self.get_settings(
            "OIDC_USERINFO_ESSENTIAL_CLAIMS",
            [],
        )

        self.OIDC_USER_SUB_FIELD = self.get_settings(
            "OIDC_USER_SUB_FIELD",
            "sub",
        )  # Default to 'sub' if not set in settings

        self.OIDC_USER_SUB_FIELD_IMMUTABLE = self.get_settings(
            "OIDC_USER_SUB_FIELD_IMMUTABLE",
            not settings.OIDC_FALLBACK_TO_EMAIL_FOR_IDENTIFICATION,
        )  # Default to False if email fallback is allowed

    def get_token(self, payload):
        """
        Return token object as a dictionary.
        Store the value to extract the refresh token in the `authenticate` method.
        """
        self._token_info = super().get_token(payload)
        return self._token_info

    def authenticate(self, request, **kwargs):
        """Authenticate a user based on the OIDC code flow."""
        user = super().authenticate(request, **kwargs)

        if user is not None:
            # Then the user successfully authenticated
            store_oidc_refresh_token(request.session, self._token_info.get("refresh_token"))
            request.session["oidc_sid"] = self._token_info.get("sid")

        return user

    def get_extra_claims(self, user_info):
        """
        Return extra claims from user_info.

        Args:
          user_info (dict): The user information dictionary.

        Returns:
          dict: A dictionary of extra claims.

        """
        return {
            # Get user's full name from OIDC fields defined in settings
            "name": self.compute_full_name(user_info),
        }

    def post_get_or_create_user(self, user, claims, is_new_user):
        """
        Post-processing after user creation or retrieval.

        Args:
          user (User): The user instance.
          claims (dict): The claims dictionary.
          is_new_user (bool): Indicates if the user was newly created.

        Returns:
        - None

        """

    def get_userinfo(self, access_token, id_token, payload):
        """
        Return user details dictionary.

        Args:
          access_token (str): The access token.
          id_token (str): The id token (unused).
          payload (dict): The token payload (unused).

        Note: The id_token and payload parameters are unused in this implementation,
        but were kept to preserve base method signature.

        Note: It handles signed and/or encrypted UserInfo Response. It is required by
        Agent Connect, which follows the OIDC standard. It forces us to override the
        base method, which deal with 'application/json' response.

        Note: Since mozilla-django-oidc >= 5.0.0, we could rely on the original method
        if we want to get rid of `self.OIDC_OP_USER_ENDPOINT_FORMAT`.

        Returns:
          dict: User details dictionary obtained from the OpenID Connect user endpoint.

        """
        user_response = requests.get(
            self.OIDC_OP_USER_ENDPOINT,
            headers={"Authorization": f"Bearer {access_token}"},
            verify=self.get_settings("OIDC_VERIFY_SSL", True),
            timeout=self.get_settings("OIDC_TIMEOUT", None),
            proxies=self.get_settings("OIDC_PROXY", None),
        )
        user_response.raise_for_status()

        _expected_format = self.OIDC_OP_USER_ENDPOINT_FORMAT
        if self.OIDC_OP_USER_ENDPOINT_FORMAT == OIDCUserEndpointFormat.AUTO:
            # In auto mode, we check the content type of the response to determine
            # the expected format.
            content_type, _params = parse_header_parameters(user_response.headers.get("Content-Type", ""))
            if content_type.lower() == "application/jwt":
                _expected_format = OIDCUserEndpointFormat.JWT
            else:
                _expected_format = OIDCUserEndpointFormat.JSON

        if _expected_format == OIDCUserEndpointFormat.JWT:
            try:
                userinfo = self.verify_token(user_response.text)
            except (UnicodeDecodeError, jwt.DecodeError) as exc:
                # Since mozilla-django-oidc >= 5.0.0, PyJWT is used, which raises jwt.DecodeError
                raise SuspiciousOperation("User info response was not valid JWT") from exc
        else:
            try:
                userinfo = user_response.json()
            except (JSONDecodeError, UnicodeDecodeError) as exc:
                raise SuspiciousOperation("User info response was not valid JSON") from exc

        return userinfo

    def verify_claims(self, claims):
        """
        Verify the presence of essential claims and the "sub" (which is mandatory as defined
        by the OIDC specification) to decide if authentication should be allowed.
        """
        essential_claims = set(self.OIDC_USERINFO_ESSENTIAL_CLAIMS) | {"sub"}
        missing_claims = [claim for claim in essential_claims if claim not in claims]

        if missing_claims:
            logger.error("Missing essential claims: %s", ", ".join(missing_claims))
            return False

        return True

    def get_or_create_user(self, access_token, id_token, payload):
        """
        Return a User based on userinfo. Create a new user if no match is found.

        Args:
          access_token (str): The access token.
          id_token (str): The ID token.
          payload (dict): The user payload.

        Returns:
          User: An existing or newly created User instance.

        Raises:
          Exception: Raised when user creation is not allowed and no existing user is found.

        """
        _user_created = False
        user_info = self.get_userinfo(access_token, id_token, payload)

        if not self.verify_claims(user_info):
            msg = "Claims verification failed"
            raise SuspiciousOperation(msg)

        sub = user_info["sub"]
        if not sub:
            raise SuspiciousOperation("User info contained no recognizable user identification")

        email = user_info.get("email")

        claims = {
            self.OIDC_USER_SUB_FIELD: sub,
            "email": email,
        }
        claims.update(**self.get_extra_claims(user_info))

        # if sub is absent, try matching on email
        user = self.get_existing_user(sub, email)

        if user:
            if not user.is_active:
                raise SuspiciousOperation(_("User account is disabled"))
            self.update_user_if_needed(user, claims)

        elif self.get_settings("OIDC_CREATE_USER", True):
            user = self.create_user(claims)
            _user_created = True

        self.post_get_or_create_user(user, claims, _user_created)
        return user

    def create_user(self, claims):
        """Return a newly created User instance."""
        sub = claims.get(self.OIDC_USER_SUB_FIELD)
        if sub is None:
            raise SuspiciousOperation(_("Claims contained no recognizable user identification"))

        logger.info("Creating user %s", sub)

        user = self.UserModel(**claims)
        user.set_unusable_password()
        user.save()

        return user

    def compute_full_name(self, user_info):
        """Compute user's full name based on OIDC fields in settings."""
        name_fields = self.OIDC_USERINFO_FULLNAME_FIELDS
        full_name = " ".join(user_info[field] for field in name_fields if user_info.get(field))
        return full_name or None

    def get_existing_user(self, sub, email):
        """Fetch existing user by sub or email."""
        try:
            return self.UserModel.objects.get(**{self.OIDC_USER_SUB_FIELD: sub})
        except self.UserModel.DoesNotExist:
            if email and settings.OIDC_FALLBACK_TO_EMAIL_FOR_IDENTIFICATION:
                try:
                    return self.UserModel.objects.get(email=email)
                except self.UserModel.DoesNotExist:
                    pass
        return None

    def update_user_if_needed(self, user, claims):
        """Update user claims if they have changed."""
        updated_claims = {}
        for key in claims:
            if not hasattr(user, key):
                continue

            claim_value = claims.get(key)
            if claim_value and claim_value != getattr(user, key):
                if key == self.OIDC_USER_SUB_FIELD and getattr(user, key) and self.OIDC_USER_SUB_FIELD_IMMUTABLE:
                    logger.warning(
                        "Attempt to change immutable field '%s' for user %s: %s -> %s",
                        key,
                        user.pk,
                        getattr(user, key),
                        claim_value,
                    )
                    continue
                if key != self.OIDC_USER_SUB_FIELD:
                    logger.info(
                        "Update '%s' field for user %s: %s -> %s",
                        key,
                        user.pk,
                        getattr(user, key),
                        claim_value,
                    )
                setattr(user, key, claim_value)
                updated_claims[key] = claim_value

        if updated_claims:
            user.save(update_fields=tuple(updated_claims.keys()))



"""Declare and configure choices for application managing accesses."""

from django.db.models import TextChoices
from django.utils.translation import gettext_lazy as _


class PriorityTextChoices(TextChoices):
    """
    Class inherits from Django's TextChoices and provides a method to get the priority
    of a given value based on its position in the class.
    """

    @classmethod
    def get_priority(cls, role):
        """Return the priority of the given role based on its order in the class."""
        members = list(cls.__members__.values())
        return members.index(role) + 1 if role in members else 0

    @classmethod
    def max(cls, *roles):
        """
        Return the highest-priority role among the given roles, using get_priority().
        If no valid roles are provided, returns None.
        """
        valid_roles = [role for role in roles if cls.get_priority(role) is not None]
        if not valid_roles:
            return None
        return max(valid_roles, key=cls.get_priority)


class LinkRoleChoices(PriorityTextChoices):
    """Define the possible roles a link can offer on a document."""

    READER = "reader", _("Reader")  # Can read
    EDITOR = "editor", _("Editor")  # Can read and edit


class RoleChoices(PriorityTextChoices):
    """Define the possible roles a user can have in a resource."""

    READER = "reader", _("Reader")  # Can read
    EDITOR = "editor", _("Editor")  # Can read and edit
    ADMIN = "administrator", _("Administrator")  # Can read, edit, delete and share
    OWNER = "owner", _("Owner")


PRIVILEGED_ROLES = [RoleChoices.ADMIN, RoleChoices.OWNER]


class LinkReachChoices(PriorityTextChoices):
    """Define types of access for links."""

    RESTRICTED = (
        "restricted",
        _("Restricted"),
    )  # Only users with a specific access can read/edit the document
    AUTHENTICATED = (
        "authenticated",
        _("Authenticated"),
    )  # Any authenticated user can access the document
    PUBLIC = "public", _("Public")  # Even anonymous users can access the document

    @classmethod
    def get_select_options(cls, link_reach, link_role):
        """
        Determine the valid select options for link reach and link role depending on the
        ancestors' link reach/role given as arguments.

        Returns:
            Dictionary mapping possible reach levels to their corresponding possible roles.

        """
        return {
            reach: [
                role
                for role in LinkRoleChoices.values
                if LinkRoleChoices.get_priority(role) >= LinkRoleChoices.get_priority(link_role)
            ]
            if reach != cls.RESTRICTED
            else None
            for reach in cls.values
            if LinkReachChoices.get_priority(reach) >= LinkReachChoices.get_priority(link_reach)
        }


def get_equivalent_link_definition(ancestors_links):
    """
    Return the (reach, role) pair.
        1. Highest reach.
        2. Highest role among links having that reach.
    """
    if not ancestors_links:
        return {"link_reach": None, "link_role": None}

    # 1) Find the highest reach
    max_reach = max(
        ancestors_links,
        key=lambda link: LinkReachChoices.get_priority(link["link_reach"]),
    )["link_reach"]

    # 2) Among those, find the highest role (ignore role if RESTRICTED)
    if max_reach == LinkReachChoices.RESTRICTED:
        max_role = None
    else:
        max_role = max(
            (link["link_role"] for link in ancestors_links if link["link_reach"] == max_reach),
            key=LinkRoleChoices.get_priority,
        )

    return {"link_reach": max_reach, "link_role": max_role}



"""malware detection module."""

from django.utils.functional import LazyObject

from .handler import MalwareDetectionHandler


class DefaultMalwareDetection(LazyObject):
    """Lazy object to handle the malware detection backend."""

    def _setup(self):
        """Configure the malware detection backend."""
        self._wrapped = malware_detection_handler()


malware_detection_handler = MalwareDetectionHandler()
malware_detection = DefaultMalwareDetection()



"""Management command to check for pending analyses and trigger them if needed."""

from django.core.management.base import BaseCommand

from lasuite.malware_detection import MalwareDetectionHandler


class Command(BaseCommand):
    """Management command to check for pending analyses and trigger them if needed."""

    help = "Call, for the active backend, the method launch_next_analysisin order to not have pending analysis blocked."

    def handle(self, *args, **options):
        """Handle the command."""
        backend = MalwareDetectionHandler()()
        backend.launch_next_analysis()



"""Brevo marketing automation integration."""

import logging
from urllib.parse import quote_plus

import requests

from lasuite.marketing.backends import ContactData
from lasuite.marketing.exceptions import ContactCreationError

from .base import BaseBackend

logger = logging.getLogger(__name__)


class BrevoBackend(BaseBackend):
    """
    Brevo marketing automation integration.

    Handles:
    - Contact management and segmentation
    - Marketing campaigns and automation
    - Email communications
    """

    def __init__(self, api_key: str, api_contact_list_ids: list[int], api_contact_attributes: dict | None = None):
        """Configure the Brevo backend."""
        self._api_key = api_key
        self.api_contact_attributes = api_contact_attributes or {}
        self.api_contact_list_ids = api_contact_list_ids

    def create_or_update_contact(self, contact_data: ContactData, timeout: int = None) -> dict:
        """
        Create or update a Brevo contact.

        Args:
            contact_data: Contact information and attributes
            timeout: API request timeout in seconds

        Returns:
            dict: Brevo API response

        Raises:
            ContactCreationError: If contact creation fails
            ImproperlyConfigured: If required settings are missing

        Note:
            Contact attributes must be pre-configured in Brevo.
            Changes to attributes can impact existing workflows.

        """
        # First try to retrieve the contact by email
        try:
            email = quote_plus(contact_data.email)
            url = f"https://api.brevo.com/v3/contacts/{email}"
            response = requests.get(
                url, params={"identifierType": "email_id"}, headers={"api-key": self._api_key}, timeout=timeout or 10
            )
            response.raise_for_status()
            contact = response.json()
        except requests.RequestException:
            pass
        else:
            # Add the list_ids from the contact in the contact_data
            list_ids = contact.get("listIds", [])
            contact_data.list_ids = (contact_data.list_ids or []) + list_ids

        attributes = {
            **self.api_contact_attributes,
            **(contact_data.attributes or {}),
        }

        # Use a set to avoid duplicates
        list_ids = set((contact_data.list_ids or []) + self.api_contact_list_ids)

        payload = {
            "email": contact_data.email,
            "updateEnabled": contact_data.update_enabled,
            "listIds": list(list_ids),
            "attributes": attributes,
        }

        print(payload)

        try:
            response = requests.post(
                "https://api.brevo.com/v3/contacts",
                json=payload,
                headers={"api-key": self._api_key},
                timeout=timeout or 10,
            )
            response.raise_for_status()
        except requests.RequestException as err:
            raise ContactCreationError("Failed to create contact in Brevo") from err

        if response.status_code == requests.codes.created:
            return response.json()

        return {}



"""Dummy marketing backend."""

from lasuite.marketing.backends import ContactData

from .base import BaseBackend


class DummyBackend(BaseBackend):
    """Dummy marketing backend doing nothing."""

    def create_or_update_contact(self, contact_data: ContactData, timeout: int = None) -> dict:
        """Create or update a contact."""
        return {}



"""Resource Server Authentication."""

import base64
import binascii
import contextlib
import logging
from functools import cache

from django.conf import settings
from django.core.exceptions import ImproperlyConfigured, SuspiciousOperation
from django.utils.module_loading import import_string
from mozilla_django_oidc.contrib.drf import OIDCAuthentication
from mozilla_django_oidc.utils import parse_www_authenticate_header
from requests.exceptions import HTTPError
from rest_framework import exceptions
from rest_framework.status import HTTP_401_UNAUTHORIZED

from .backend import ResourceServerBackend, ResourceServerImproperlyConfiguredBackend

logger = logging.getLogger(__name__)


@cache
def get_resource_server_backend() -> type[ResourceServerBackend]:
    """Return the resource server backend class based on the settings."""
    return import_string(settings.OIDC_RS_BACKEND_CLASS)


class ResourceServerAuthentication(OIDCAuthentication):
    """
    Authenticate clients using the token received from the authorization server.

    We still inherit from OIDCAuthentication for the basic token extraction,
    but the authenticate method is fully overridden.
    """

    def __init__(self):
        """Require authentication to be configured in order to instantiate."""
        try:
            super().__init__(backend=get_resource_server_backend()())
        except ImproperlyConfigured as err:
            message = "Resource Server authentication is disabled"
            logger.debug("%s. Exception: %s", message, err)
            self.backend = ResourceServerImproperlyConfiguredBackend()

    def get_access_token(self, request):
        """
        Retrieve and decode the access token from the request.

        This method overrides the 'get_access_token' method from the parent class,
        to support service providers that would base64 encode the bearer token.
        """
        access_token = super().get_access_token(request)

        with contextlib.suppress(binascii.Error, TypeError, UnicodeDecodeError):
            access_token = base64.b64decode(access_token, validate=True).decode("utf-8")

        return access_token

    def authenticate(self, request):
        """
        Authenticate the request and return a tuple of (user, token) or None.

        We fully override the 'authenticate' method from the parent class:
         - to be able to introspect the access token earlier (doing it in the
           get_user method would be too late);
         - to store the introspected token audience inside the request to allow the
           views to use it later (for permission restriction).

        The implementation is still highly inspired from the parent class.
        """
        access_token = self.get_access_token(request)

        if not access_token:
            # Defer to next authentication backend
            return None

        # Custom addition: introspect the access token to ensure it's valid
        # and retrieve user info.
        try:
            user_info = self.backend.get_user_info_with_introspection(access_token)
        except HTTPError as exc:
            resp = exc.response

            # If the oidc provider returns 401, it means the token is invalid or that
            # the introspection is not allowed.
            # In that case, we want to return the upstream error message (which
            # we can get from the www-authentication header) in the response.
            if resp.status_code == HTTP_401_UNAUTHORIZED and "www-authenticate" in resp.headers:
                data = parse_www_authenticate_header(resp.headers["www-authenticate"])
                raise exceptions.AuthenticationFailed(
                    data.get("error_description", "no error description in www-authenticate")
                ) from exc

            # for all other http errors, just re-raise the exception.
            raise

        try:
            user = self.backend.get_or_create_user(access_token, None, user_info)

        except SuspiciousOperation as exc:
            logger.info("Login failed: %s", exc)
            raise exceptions.AuthenticationFailed("Login failed") from exc

        if not user:
            msg = "Login failed: No user found for the given access token."
            raise exceptions.AuthenticationFailed(msg)

        # Custom addition: store the token audience in the request
        # Note: at this stage, the request is a "drf_request" object
        request.resource_server_token_audience = self.backend.token_origin_audience

        return user, user_info



"""Enum for OIDC login."""

from enum import StrEnum


# OIDC_OP_USER_ENDPOINT allowed values
class OIDCUserEndpointFormat(StrEnum):
    """Enum for OIDC OP User Endpoint."""

    JWT = "jwt"
    JSON = "json"
    AUTO = "auto"



# Generated by Django 5.2.3 on 2025-11-28 14:16

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('malware_detection', '0002_alter_malwaredetection_parameters'),
    ]

    operations = [
        migrations.AddField(
            model_name='malwaredetection',
            name='error_code',
            field=models.IntegerField(blank=True, help_text='error code for the detection', null=True),
        ),
        migrations.AddField(
            model_name='malwaredetection',
            name='error_msg',
            field=models.TextField(blank=True, help_text='error message for the detection'),
        ),
        migrations.AlterField(
            model_name='malwaredetection',
            name='status',
            field=models.CharField(choices=[('pending', 'Pending'), ('processing', 'Processing'), ('failed', 'Failed')], default='pending', help_text='status of the detection', max_length=255),
        ),
        migrations.AddField(
            model_name='malwaredetection',
            name='backend',
            field=models.TextField(blank=True, help_text='backend used for the detection'),
        ),
    ]



"""Marketing module."""

from django.utils.functional import LazyObject

from .handler import MarketingHandler


class DefaultMarketing(LazyObject):
    """Lazy object to handle the marketing backend."""

    def _setup(self):
        """Configure the marketing backend."""
        self._wrapped = marketing_handler()


marketing_handler = MarketingHandler()
marketing = DefaultMarketing()



"""Marketing backend base module."""

from abc import ABC, abstractmethod

from lasuite.marketing.backends import ContactData


class BaseBackend(ABC):
    """Base class for all marketing backends."""

    @abstractmethod
    def create_or_update_contact(self, contact_data: ContactData, timeout: int = None) -> dict:
        """
        Create or update a contact.

        Args:
            contact_data: Contact information and attributes
            timeout: API request timeout in seconds

        Returns:
            dict: Service response

        Raises:
            ContactCreationError: If contact creation fails

        """



"""Email related tools."""

from email.errors import HeaderParseError
from email.headerregistry import Address


def get_domain_from_email(email: str | None) -> str | None:
    """Extract domain from email."""
    try:
        address = Address(addr_spec=email)
        if len(address.username) > 64 or len(address.domain) > 255:  # noqa: PLR2004
            # Simple length validation using the RFC 5321 limits
            return None
        if not address.domain:
            # If the domain is empty, return None
            return None
        return address.domain
    except (ValueError, AttributeError, IndexError, HeaderParseError):
        return None



"""marketing backendhandler."""

from django.conf import settings
from django.core.exceptions import ImproperlyConfigured
from django.utils.functional import cached_property
from django.utils.module_loading import import_string

from lasuite.marketing.exceptions import MarketingInvalidBackendError


class MarketingHandler:
    """Marketing handler managing the backend instantiation."""

    def __init__(self, backend=None):
        """Initialize the marketing handler."""
        # backend is an optional dict of marketing backend definitions
        # (structured like settings.LASUITE_MARKETING).
        self._backend = backend
        self._marketing = None

    @cached_property
    def backend(self):
        """Put in cache the backend properties from the settings."""
        if self._backend is None:
            try:
                self._backend = settings.LASUITE_MARKETING.copy()
            except AttributeError as e:
                raise ImproperlyConfigured("settings.LASUITE_MARKETING is not configured") from e
        return self._backend

    def __call__(self):
        """Create if not existing the backend and then return it."""
        if self._marketing is None:
            self._marketing = self.create_marketing(self.backend)
        return self._marketing

    def create_marketing(self, params):
        """Instantiate and configure the marketing backend."""
        params = params.copy()
        backend = params.pop("BACKEND")
        parameters = params.pop("PARAMETERS", {})
        try:
            klass = import_string(backend)
        except ImportError as e:
            raise MarketingInvalidBackendError(f"Could not find backend {backend!r}: {e}") from e
        return klass(**parameters)



"""Marketing exceptions module."""


class MarketingError(Exception):
    """Base exception for all marketing exceptions."""


class MarketingInvalidBackendError(MarketingError):
    """Exception raised when the backend is invalid."""


class ContactCreationError(MarketingError):
    """Exception raised when the contact creation fails."""



# Generated by Django 5.2.3 on 2025-08-28 13:56

import lasuite.malware_detection.models
from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('malware_detection', '0001_initial'),
    ]

    operations = [
        migrations.AlterField(
            model_name='malwaredetection',
            name='parameters',
            field=models.JSONField(blank=True, default=dict, encoder=lasuite.malware_detection.models.JsonUUIDEncoder, help_text='parameters for the detection', null=True),
        ),
    ]
